import java.awt.event.KeyEvent;
import java.util.ArrayList; 

/**
 * Ein Tastaturecho, das die eingegeben Zeichen auf der
 * Leinwand darstellt.
 * 
 * @author Bernd Gramlich 
 * @version 2013-03-01
 */
public class TastaturEcho implements ZahlErzeuger
{
    /*
     * Die Darstellung der letzten eingegebenen Zeichen auf
     * der Leinwand.
     */
    private Beschriftung echo;

    /*
     * Der Inhalt des Echos als Zeichenkette.
     */
    private String inhalt;

    /*
     * Die Anzahl der im Echo dargestellten Zeichen.
     */
    private static final int laenge = 20;

    /*
     * Die letzte eingegebene Ziffer.
     */
    private int ziffer;

    /*
     * Die Liste unserer Zahlbeobachter.
     */
    private ArrayList<ZahlBeobachter> zahlBeobachter;

    /**
     * Konstruktor.
     * Erzeuge das Echo mit leerem Inhalt.
     */
    public TastaturEcho ()
    {
        /*
         * Erzeuge ein Echo mit leerem Inhalt.
         */
        inhalt = "";

        echo = new Beschriftung ();
        echo.setzeInhalt (inhalt);
        echo.setzeSchriftgroesse (40);
        echo.setzeFarbe ("grau");
        echo.setzePosition (10, 500);
        echo.macheSichtbar ();
        
        /*
         * Nenne anfangs als Ziffer -1.
         */
        ziffer = -1;
        
        /*
         * Erzeuge die Liste unserer Zahlbeobachter.
         */
        zahlBeobachter = new ArrayList<ZahlBeobachter> ();
    }

    /**
     * Starte das Echo.
     * Das Echo gibt Buchstaben und Ziffern aus der Tastatureingabe
     * wieder. Es endet, wenn man die Eingabetaste drckt. Andere
     * Tasten werden ignoriert.
     */
    public void starte ()
    {
        /*
         * Hilfsvariablen:
         * anzeige:       Die Leinwand, von der wir die Tastatureingabe lesen und
         *                auf der wir das Echo ausgeben.
         * taste:         Die letzte Taste, die auf der Leinwand gedrckt wurde
         *                (als Tastaturereignis).
         * buchstabe:     Der Buchstabe (oder das Zeichen) auf der gedrckten Taste.
         * aufhoeren:     Will der Benutzer aufhren?
         */
        Leinwand anzeige = Leinwand.gibLeinwand ();
        KeyEvent taste;
        char buchstabe;
        boolean aufhoeren;       

        /*
         * Lsche alle Tasten, die vor Echobeginn gedrckt wurden.
         */
        while ((taste = anzeige.tastendruck ()) != null);
        
        /*
         * Lsche den Inhalt des Echos.
         */
        inhalt = "";
        echo.setzeInhalt (inhalt);

        /*
         * Wir lesen mindestens ein Zeichen ein.
         */
        aufhoeren = false;

        /*
         * Solange der Benutzer Zeichen eingeben will ...
         */
        while (!aufhoeren)
        {
            /*
             * Warte, bis der Benutzer eine Taste drckt.
             */
            while ((taste = anzeige.tastendruck ()) == null);
            
            /*
             * Bestimme den Buchstaben, der auf der Taste steht.
             */
            buchstabe = taste.getKeyChar ();
            
            /*
             * Wenn tatschlich ein Buchstabe oder eine Ziffer vorliegt,
             * bernimm die Tastatureingabe ins Echo
             */
            if (('a' <= buchstabe && buchstabe <= 'z')
                || ('A' <= buchstabe && buchstabe <= 'Z')
                || ('0' <= buchstabe && buchstabe <= '9'))
            {
                if (inhalt.length () < laenge)
                {
                    inhalt = inhalt + buchstabe;
                }
                else
                {
                    inhalt = inhalt.substring (1) + buchstabe;
                }
                
                echo.setzeInhalt (inhalt);
            }
            
            /*
             * Wenn eine Ziffer vorliegt, speichere sie und benachrichtige
             * die Beobachter.
             */
            if ('0' <= buchstabe && buchstabe <= '9')
            {
                ziffer = buchstabe - '0';
                
                benachrichtigeZahlbeobachter ();
            }
            
            /*
             * Spiele weiter auer bei der Eingabetaste.
             */            
            if (KeyEvent.VK_ENTER == buchstabe)
            {
                aufhoeren = true;
            }
        }
    }
    
    /**
     * Nenne die aktuelle Zahl.
     * 
     * @return Die aktuelle Zahl.
     */
    public int gibZahl ()
    {
        return ziffer;
    }

    /**
     * Speichere einen Zahlbeobachter.
     * 
     * @param b Der Zahlbeobachter.
     */
    public void speichereAlsBeobachter (ZahlBeobachter b)
    {
        zahlBeobachter.add (b);
    }

    /**
     * Loesche einen Zahlbeobachter.
     * 
     * @param b Der Zahlbeobachter.
     */
    public void loescheAlsBeobachter (ZahlBeobachter b)
    {
        zahlBeobachter.remove (b);
    }

    /**
     * Benachrichtige alle Zahlbeobachter, dass sich die Zahl
     * geaendert hat.
     */
    private void benachrichtigeZahlbeobachter ()
    {
        for (ZahlBeobachter b: zahlBeobachter)
        {
            b.aktualisiereZahl ();
        }
    }
}